﻿// Пример кода программы, которая выполняет экспорт списка объектов из ADVANTA в Google-таблицу.
// Для работы программы должен быть создан "проект" в сервисе "Google Cloud Console". 
// В нём необходимо подключить библиотеку API Google Sheets и настроить "сервисный аккаунт" для доступа.
// Данные для авторизации сервисного аккаунта необходимо скачать в файл и поместить в каталог программы. Имя файла по умолчанию: service-account.json.  
// На стенде ADVANTA должен быть LINQ-запрос, возвращающий список объектов. Реквизиты Id и Name должны быть обязательно. 
// Можно добавить в запрос реквизиты «Процент завершения» (PercentComplete), «Статус» (Status) и «Приоритет» (Priority), пример сможет их отобразить. 
// Если потребуется выгрузить больше реквизитов, нужно добавить их в сам LINQ-запрос, в класс «AdvantaObject» и в метод «WriteAdvantaObjectsToSheet» в коде примера.
// Данные для подключения в ADVANTA надо настроить в коде: адрес сервера ADVANTA, логин/пароль, ключ LINQ-запроса и количество строк, которые программа из него получит.
// Также нужно указать идентификатор Google-таблицы и название листа.

using Google.Apis.Auth.OAuth2;  
using Google.Apis.Services;  
using Google.Apis.Sheets.v4;  
using Google.Apis.Sheets.v4.Data;  
using System;  
using System.Collections.Generic;   
using System.IO;  
using System.Net.Http;  
using System.Text;  
using System.Threading.Tasks;  
using Newtonsoft.Json;

// Класс для создания объектов из LINQ-запроса ADVANTA
// Примечание: LINQ-запрос GetAllObjects возвращает только Id и Name. Остальные поля надо добавить в запрос, если они нужны.
public class AdvantaObject  
{  
    public required string Id { get; set; }  
    public required string Name { get; set; }  
    public int PercentComplete { get; set; }
    public int Status { get; set; }  
    public int Priority { get; set; } 
}

// Основной класс программы
class Program  
{  
    // Адрес сервера ADVANTA
    private static string a2serverUrl = "https://pmo.a2nta.ru";

    // Логин пользователя для подключения к ADVANTA
    private static string a2login = "admin";

    // Пароль пользователя для подключения к ADVANTA
    private static string a2password = "123456";

    // Ключ LINQ-запроса в ADVANTA 
    private static string a2dataSourceKey = "GetAllObjects"; 

    // Количество объектов, получаемых из LINQ-запроса в ADVANTA (при большом количестве данные будут загружаться долго)
    private static int a2numOfObjects = 7; 

    // Имя файла с данными для авторизации в Google
    private static string gFileName = "service-account.json";
    
    // Идентификатор Google-таблицы
    private static string sheetId = "1v5lqduyizJtf9lliUkfFEjGo4j61eNX3nwV-FkZo5DU";

    // Имя листа в Google-таблице
    private static string sheetName = "Лист1";
    
    static async Task Main(string[] args)  
    {  
        // Формируем данные для подключения в Google API  
        var service = CreateSheetsService();  

        try  
        {  
            // Попытка получения доступа к Google-таблице 
            Console.WriteLine("Проверяю доступ к Google-таблице.");  
            var range = $"{sheetName}!A1:A2"; // Укажите диапазон, который хотите проверить  
            var request = service.Spreadsheets.Values.Get(sheetId, range);  
            var response = request.Execute();  

            if (response != null)  
            {  
                Console.WriteLine("Доступ к Google-таблице успешен.");  

                // Вызов метода и получение куки ADVANTA для дальнейшего выполнения LINQ-запроса 
                string cookie = await AuthorizeInAdvanta();

                // Если авторизация успешна, выполняем следующий запрос  
                if (!string.IsNullOrEmpty(cookie))  
                {  
                    Console.WriteLine("Авторизация в ADVANTA пройдена успешно."); 

                    var advantaObjects = await GetObjectsList(cookie); // Получаем объекты из ADVANTA  

                    if (advantaObjects != null && advantaObjects.Count > 0) // Проверяем, что список не пуст  
                    {  
                        Console.WriteLine("Список объектов ADVANTA успешно получен (" + advantaObjects.Count + " шт).");
                        
                        // Очистка таблицы  
                        ClearSheet(service, sheetId, sheetName);  
                        Console.WriteLine("Таблица успешно очищена.");

                        // Записываем новые строки в таблицу
                        await WriteAdvantaObjectsToSheet(service, sheetId, sheetName, advantaObjects); // Записываем объекты в таблицу  
                    }  
                    else  
                    {  
                        Console.WriteLine("Полученные объекты пусты. Запись в Google-таблицу не выполнена.");  
                    }    
                }    
            }  
            else  
            {  
                Console.WriteLine("Google-таблица не доступна!");  
            }  
        }  
        catch (Exception ex)  
        {  
            Console.WriteLine("Ошибка доступа: " + ex.Message);  
        }   
    }  

    // Метод создаёт объект для авторизации в проекте Google Cloud console с подключенным Google Sheets API.  
    static SheetsService CreateSheetsService()  
    {  
        Console.WriteLine("Читаю JSON-файл с данными для авторизации."); 
        
        GoogleCredential credential;  

        // Загружаем файл с данными для авторизации
        using (var stream = new FileStream(gFileName, FileMode.Open, FileAccess.Read))  
        {  
            credential = GoogleCredential.FromStream(stream)  
                .CreateScoped(SheetsService.Scope.Spreadsheets);  
        }  

        // Возвращаем объект с данными для авторизации
        return new SheetsService(new BaseClientService.Initializer()  
        {  
            HttpClientInitializer = credential,  
            ApplicationName = "Google Sheets API Example",  
        });  
    }  

    // Метод очищает указанную Google-таблицу
    static void ClearSheet(SheetsService service, string spreadsheetId, string sheetName)  
    {  
        Console.WriteLine("Начинаю очистку Google-таблицы"); 
        // Создаем запрос для очистки значений в указанном диапазоне  
        var clearRequest = new ClearValuesRequest();  
        var request = service.Spreadsheets.Values.Clear(clearRequest, spreadsheetId, $"{sheetName}!A1:Z"); // Очистка диапазона от A1 до Z  
        request.Execute();  
    }  

    // Метод авторизации в ADVANTA для выполнеия запросов web-api  
    private static async Task<string> AuthorizeInAdvanta()   
    {  
        Console.WriteLine("Начинаю авторизацию в ADVANTA"); 
        
        // Переменная, для возврата результата работы метода
        string result = ""; 
        
        using (var httpClient = new HttpClient())  
        {  
            // Формируем данные для запроса  
            var loginData = new  
            {  
                Login = a2login,  
                Password = a2password  
            };  

            var content = new StringContent(JsonConvert.SerializeObject(loginData), Encoding.UTF8, "application/json");  

            // Выполняем POST-запрос  
            var response = await httpClient.PostAsync($"{a2serverUrl}/api/auth/login", content);  

            if (response.IsSuccessStatusCode)  
            {  
                // Извлекаем куки из заголовка Set-Cookie  
                IEnumerable<string>? setCookieHeaders;  
                if (response.Headers.TryGetValues("Set-Cookie", out setCookieHeaders))  
                {  
                    // Берем первое значение куки и отсекаем дополнительную информацию  
                    var fullSetCookie = setCookieHeaders.FirstOrDefault()?.Trim();  
                    if (fullSetCookie != null)  
                    {  
                        // Берём только значение самого куки, игнорируя дополнительные параметры для браузера
                        var semicolonIndex = fullSetCookie.IndexOf(';');  
                        var cookie = semicolonIndex > 0 ? fullSetCookie.Substring(0, semicolonIndex) : fullSetCookie;
                        Console.WriteLine("Куки успешно извлечены: " + cookie); 
                        result = cookie;
                    }  
                }  
                else  
                {  
                    Console.WriteLine("Куки не найдены в заголовке ответа."); 
                    result = string.Empty; 
                }  
            }  
            else  
            {  
                Console.WriteLine("Ошибка авторизации: " + response.ReasonPhrase);  
                result = string.Empty;
            }  
        }  

        return result;
    }

    // Метод получает список объектов из ADVANTA, через LINQ-запрос 
    public static async Task<List<AdvantaObject>> GetObjectsList(string cookie)  
    {  
        Console.WriteLine("Начинаю вызов LINQ-запроса в ADVANTA"); 
        
        var advantaObjects = new List<AdvantaObject>(); // Инициализируем пустой список

        using (var httpClient = new HttpClient())  
        {  
            // Устанавливаем хост  
            httpClient.BaseAddress = new Uri(a2serverUrl);  
            
            // Формируем данные для запроса  
            var queryData = new  
            {  
                DataSourceKey = a2dataSourceKey,  
                PageSize = a2numOfObjects  
            };  

            var content = new StringContent(JsonConvert.SerializeObject(queryData), Encoding.UTF8, "application/json");  

            // Добавляем заголовок куки  
            if (!string.IsNullOrEmpty(cookie))  
            {  
                httpClient.DefaultRequestHeaders.Add("Cookie", cookie);  
            }  

            // Выполняем POST-запрос на указанный URL  
            var response = await httpClient.PostAsync("/api/queries/get", content);  

            if (response.IsSuccessStatusCode)  
            {  
                var responseData = await response.Content.ReadAsStringAsync();  

                // Десериализуем ответ в список объектов  
                var deserializedObjects = JsonConvert.DeserializeObject<List<AdvantaObject>>(responseData);  
                if (deserializedObjects != null)  
                {  
                    advantaObjects = deserializedObjects; // Присваиваем либо восстановленный список, либо пустой  
                } 
            }  
            else  
            {  
                Console.WriteLine("Ошибка получения списка объектов: " + response.ReasonPhrase);  
            }  
        }  

        return advantaObjects; // Всегда возвращаем ненулевой список
    }

    // Записывает объекты в ячейки Google-таблицы
    static async Task WriteAdvantaObjectsToSheet(SheetsService service, string spreadsheetId, string sheetName, List<AdvantaObject> advantaObjects) 
    {   
        Console.WriteLine("Начинаю запись строк в Google-таблицу."); 
        
        // Проверяем, что список объектов не null и содержит элементы  
        if (advantaObjects == null || advantaObjects.Count == 0) // Здесь обновлено на Count == 0  
        {  
            Console.WriteLine("Нет объектов для записи в таблицу.");  
            return; // Если список пуст, выходим из метода  
        }  

        // Подготавливаем данные для записи  
        var values = new List<IList<object>>  
        {  
            new List<object> { "Id", "Name", "PercentComplete", "Status", "Priority" } // Заголовки  
        };  

        foreach (var obj in advantaObjects)  
        {  
            values.Add(new List<object> { obj.Id, obj.Name, obj.PercentComplete, obj.Status, obj.Priority});  
        }  

        // Пакет значений для Google Sheets  
        var body = new Google.Apis.Sheets.v4.Data.ValueRange()  
        {  
            Values = values  
        };  

        var range = $"{sheetName}!A1"; // Определяем диапазон для записи  
        var request = service.Spreadsheets.Values.Append(body, spreadsheetId, range);  
        request.ValueInputOption = SpreadsheetsResource.ValuesResource.AppendRequest.ValueInputOptionEnum.USERENTERED;  

        try  
        {  
            var response = await request.ExecuteAsync();  
            Console.WriteLine("Данные успешно записаны в таблицу Google Sheets.");  
        }  
        catch (Exception ex)  
        {  
            Console.WriteLine("Ошибка при записи данных в Google Sheets: " + ex.Message);  
        }
    }  
}